#include <nokia_lcd.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_gpio.h>


//	   
void lcd_pin_init(void)
{
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO|lcd_RCC, ENABLE);
	//GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable, ENABLE);
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_NoJTRST  , ENABLE);

	GPIO_InitTypeDef lcd_GPIO_InitStructure;	//	    

	lcd_GPIO_InitStructure.GPIO_Pin = lcd_pin_SCLK | lcd_pin_SDA |lcd_pin_CS | lcd_pin_RST;
	lcd_GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;  // 
	lcd_GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(lcd_port, &lcd_GPIO_InitStructure);
}

void nlcd_Inverse(unsigned char mode)
{
	if (mode) lcd_write(lcd_CMD,0xA6);
	 else lcd_write(lcd_CMD,0xA7);
}

//	     
void lcd_test(void)
{
	unsigned char i;
  lcd_gotoxy(0,0);
	for(i=32;i<127;i++)
  {
		lcd_putch(i);
	}
	for(i=192;i<=224;i++)
  {
		lcd_putch(i);
	}
}

void char_5_8(char c, char x, char y)
{
	unsigned char i;
	lcd_gotoxy(x,y);
        	for ( i = 0; i < 5; i++ )
            {
				lcd_write(lcd_DATA,lcd_font[c+16][i] << 1);
            }
			lcd_write(lcd_DATA,0x00);

}

//	    2,4,8 
//	Input : col - , row - , *s -   
//	       2: 1,2,4,8
void lcd_puts_big(unsigned char col,unsigned char row,unsigned char *s)
{
	while (*s)
	{
		lcd_putch_big(col,row,*s);
		++s;
	}

}

//	    2,4,8 
//	Input : col - , row - , c -   ASCII
//	       2: 1,2,4,8
void lcd_putch_big(unsigned char col,unsigned char row,unsigned char c)
{
	unsigned char i,j,k,l,variable1,shift1,variable2,shift2,shift3,t_X,t_Y;
	unsigned int sym,sym2;
	t_X=lcd_X;
	t_Y=lcd_Y;
	c=lcd_symbol_decode(c);
	if(c==255)
	{
		return;
	}
	lcd_gotoxy(lcd_X,lcd_Y);
	variable1=0;
	shift1=8/row;
	variable2=1;
	switch(row)
	{
		case 1: {shift2=1; shift3=1; }break;
		case 2: {shift2=16;shift3=2; }break;
		case 4: {shift2=4; shift3=8; }break;
		case 8: {shift2=2; shift3=32;}break;
	}
	for(i=0;i<row;i++)
	{
		for(j=0;j<5;j++)
		{
			sym=0;
			l=1;
			for(k=variable1;k<variable1+shift1;k++)
			{
				sym+=(lcd_font[c][j]&(1<<k))*l;
				l*=shift3;
			}
			sym=sym/variable2;

			sym2=0;
			for(k=0;k<row;k++)
			{
				sym2=sym2+sym;
				sym*=2;
			}
			for(k=0;k<col;k++)
			{
				lcd_write(lcd_DATA,sym2);
			}
		}
		lcd_gotoxy(lcd_X,lcd_Y+1);
		variable1+=shift1;
		variable2*=shift2;
	}
	lcd_X=t_X+col;
	lcd_Y=t_Y;
}

void lcd_puts_inv(const char *s)
{
	while (*s)
	{
		lcd_putch_inv(*s);
		++s;
	}
}
//	 
void lcd_clear(void)
{
	unsigned int i;
	lcd_write(lcd_CMD,0x40);// Y = 0
	lcd_write(lcd_CMD,0xb0);
	lcd_write(lcd_CMD,0x10);// X = 0
	lcd_write(lcd_CMD,0x00);
	lcd_write(lcd_CMD,0xae);// disable display;
	for(i=0;i<864;i++)
	lcd_write(lcd_DATA,0x00);
	lcd_write(lcd_CMD,0xaf);// enable display;
}

//	 
// 	Input : x,y -  
void lcd_gotoxy(unsigned char x,unsigned char y)
{
	lcd_X=x;
	lcd_Y=y;
	x=x*6;
	lcd_write(lcd_CMD,(0xB0|(y&0x0F)));// Y axis initialisation: 0100 yyyy
	lcd_write(lcd_CMD,(0x00|(x&0x0F))); // X axis initialisation: 0000 xxxx ( x3 x2 x1 x0)
	lcd_write(lcd_CMD,(0x10|((x>>4)&0x07)));// X axis initialisation: 0010 0xxx  ( x6 x5 x4)
}

//	        
//	Input :  -   ASCII 
unsigned char lcd_symbol_decode(unsigned char c)
{
	if(32<=c&&c<='~')
	{
		c=c-32;
	}
	else
	{
		if(192<=c&&c<=255)
		{
			c=c-97;
		}
		else
		{
			c=255;
		}
	}
	return c;
}



//	   
//	Input :  -   ASCII 
void lcd_putch(unsigned char c)
{
	c=lcd_symbol_decode(c);
	if(c==255)
	{
		return;
	}
	lcd_write(lcd_DATA,lcd_font[c][0]);
	lcd_write(lcd_DATA,lcd_font[c][1]);
	lcd_write(lcd_DATA,lcd_font[c][2]);
	lcd_write(lcd_DATA,lcd_font[c][3]);
	lcd_write(lcd_DATA,lcd_font[c][4]);
	lcd_write(lcd_DATA,0x00);
	lcd_X++;
	if(lcd_X==16)
	{
		if(lcd_Y==7)
		{
			lcd_gotoxy(0,0);
		}
		else
		{
			lcd_gotoxy(0,lcd_Y+1);
		}
	}
}


void lcd_putch_inv(unsigned char c)
{
	c=lcd_symbol_decode(c);
	if(c==255)
	{
		return;
	}
	lcd_write(lcd_DATA,255-lcd_font[c][0]);
	lcd_write(lcd_DATA,255-lcd_font[c][1]);
	lcd_write(lcd_DATA,255-lcd_font[c][2]);
	lcd_write(lcd_DATA,255-lcd_font[c][3]);
	lcd_write(lcd_DATA,255-lcd_font[c][4]);
	lcd_write(lcd_DATA,255);
	lcd_X++;
	if(lcd_X==16)
	{
		if(lcd_Y==7)
		{
			lcd_gotoxy(0,0);
		}
		else
		{
			lcd_gotoxy(0,lcd_Y+1);
		}
	}
}


//	   
//	Input : s - 
void lcd_puts(const char *s)
{
	while (*s)
	{
		lcd_putch(*s);
		++s;
	}
}


//	 
void lcd_init(void)
{
	lcd_CS_off;
	lcd_RST_off;
	_delay_ms(25);
	lcd_RST_on;
	_delay_ms(25);
/*	lcd_write(lcd_CMD,0xE2);
	_delay_ms(10);
	lcd_write(lcd_CMD,0xAF);
	lcd_write(lcd_CMD,0xA4);
	lcd_write(lcd_CMD,0x2F);
	lcd_write(lcd_CMD,0xB0);
	lcd_write(lcd_CMD,0x10);
	lcd_write(lcd_CMD,0x00);*/

	lcd_write(lcd_CMD,0x20);//- write VOP register
	lcd_write(lcd_CMD,0x90); //  0x90 0x9f
	lcd_write(lcd_CMD,0xA4);//- all on/normal display  A4/A5
	lcd_write(lcd_CMD,0x2F);//- Power control set(charge pump on/off)
	lcd_write(lcd_CMD,0x40);//- set start row address = 0
	lcd_write(lcd_CMD,0xb0);//- set Y-address = 0
	lcd_write(lcd_CMD,0x10);//- set X-address, upper 3 bits
	lcd_write(lcd_CMD,0x0);//- set X-address, lower 4 bits
//	lcd_write(lcd_CMD,0xC8);//- mirror Y axis (about X axis)
//	lcd_write(lcd_CMD,0xa1);// Invert screen in horizontal axis
	lcd_write(lcd_CMD,0xac);//- set initial row (R0) of the display
	lcd_write(lcd_CMD,0x07);
	lcd_write(lcd_CMD,0xaf); //- display ON/OFF

	lcd_clear();
//	lcd_write(lcd_CMD,0xa7); //- invert display
	lcd_write(lcd_CMD,0xa6);//- normal display (non inverted )
	_delay_ms(500);
}


//	  
//	Input : cd -   ,  -    
void lcd_write(char cd,unsigned char c)
{
	char i;
	lcd_CS_off;
	lcd_SCLK_off;
	if(cd)
	{
		lcd_SDA_on;
	}
	else
	{
		lcd_SDA_off;
	}
	lcd_SCLK_on;
	for(i=0;i<8;i++)
	{
		lcd_SCLK_off;
		if(c & 0x80)
		{
			lcd_SDA_on;
		}
		else
		{
			lcd_SDA_off;
		}
		lcd_SCLK_on;
		c <<= 1;
		_delay_us(35); //Delay10TCYx(20);
	}
	lcd_CS_on;   //???
}
